/**
 *      Copyright (C) 2012 David M. Carr
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.bitbucket.davidm24.mongodb.aggregate;

import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;

/**
 * Utility for creating DBObject aggregate commands.
 *
 * @author David M. Carr
 * @see <a href="http://docs.mongodb.org/manual/applications/aggregation/">Aggregation Framework in MongoDB Manual</a>
 */
public class AggregateBuilder {
    public AggregateBuilder(String collectionName) {
        _pipeline = new BasicDBList();
        _command = new BasicDBObject();
        _command.put("aggregate", collectionName);
        _command.put("pipeline", _pipeline);
    }

    /**
     * Returns a new AggregateBuilder for the specified collection.  All subsequent operators will be treated as part of
     * the processing pipeline.  When you're done adding to the pipeline, call {@link #get()}.
     *
     * @param collection the collection within which to aggregate
     * @return the new builder instance
     */
    public static AggregateBuilder aggregate(DBCollection collection) {
        return aggregate(collection.getName());
    }

    /**
     * Returns a new AggregateBuilder for the specified collection.  All subsequent operators will be treated as part of
     * the processing pipeline.  When you're done adding to the pipeline, call {@link #get()}.
     *
     * @param collectionName the name of the collection within which to aggregate
     * @return the new builder instance
     */
    public static AggregateBuilder aggregate(String collectionName) {
        return new AggregateBuilder(collectionName);
    }

    private void addOperand(String op, Object value) {
        _pipeline.add(new BasicDBObject(op, value));
    }

    public AggregateBuilder project(Object object) {
        addOperand(PipelineOperators.PROJECT, object);
        return this;
    }

    public AggregateBuilder match(Object object) {
        addOperand(PipelineOperators.MATCH, object);
        return this;
    }

    /**
     * Shortcut for simple single-key, direct equality match clauses.  For more complicated uses, use
     * {@link #match(Object)}.
     *
     * @param key the field to match
     * @param value the precise value to match to
     * @return the builder
     */
    public AggregateBuilder match(String key, Object value) {
        return match(new BasicDBObject(key, value));
    }

    public AggregateBuilder limit(int count) {
        addOperand(PipelineOperators.LIMIT, count);
        return this;
    }

    public AggregateBuilder skip(int count) {
        addOperand(PipelineOperators.SKIP, count);
        return this;
    }

    public AggregateBuilder unwind(String targetField) {
        addOperand(PipelineOperators.UNWIND, targetField);
        return this;
    }

    public AggregateBuilder group(Object object) {
        addOperand(PipelineOperators.GROUP, object);
        return this;
    }

    /**
     * Shortcut for simple single-key group clauses.  For more complicated uses, use
     * {@link #group(Object)}.
     *
     * @param key the field to match
     * @param value the precise value to match to
     * @return the builder
     */
    public AggregateBuilder group(String key, Object value) {
        return group(new BasicDBObject(key, value));
    }

    public AggregateBuilder sort(Object object) {
        addOperand(PipelineOperators.SORT, object);
        return this;
    }

    /**
     * Creates a <code>DBObject</code> to be passed to the driver's command operation
     *
     * @return Returns a DBObject instance
     */
    public DBObject get() {
        return _command;
    }

    private DBObject _command;
    private BasicDBList _pipeline;
}
