package org.bitbucket.davidm24.mongodb.aggregate;

import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

/**
 * Utility for creating DBObject $group body clauses.
 *
 * @author David M. Carr
 * @see <a href="http://docs.mongodb.org/manual/applications/aggregation/">Aggregation Framework in MongoDB Manual</a>
 */
public class GroupBuilder {
    public GroupBuilder() {
        _body = new BasicDBObject();
    }

    /**
     * Returns a new GroupBuilder
     */
    public static GroupBuilder start() {
        return new GroupBuilder();
    }

    public static GroupBuilder start(String targetField) {
        return start().push(targetField);
    }

    private void addOperand(String op, Object value) {
        if (_currentKey == null) {
            throw new GroupBuilderException("No target field for operand");
        }
        if (op == null) {
            _body.put(_currentKey, value);
        } else {
            _body.put(_currentKey, new BasicDBObject(op, value));
        }
    }

    public GroupBuilder put(String key) {
        _currentKey = key;
        if (_body.get(key) == null) {
            _body.put(_currentKey, new NullObject());
        }
        return this;
    }

    public GroupBuilder to(Object object) {
        addOperand(null, object);
        return this;
    }

    public GroupBuilder sum(int num) {
        addOperand(GroupOperators.SUM, num);
        return this;
    }

    public GroupBuilder sum(String targetField) {
        addOperand(GroupOperators.SUM, targetField);
        return this;
    }

    public GroupBuilder addToSet(String targetField) {
        addOperand(GroupOperators.ADD_TO_SET, targetField);
        return this;
    }

    public GroupBuilder first(String targetField) {
        addOperand(GroupOperators.FIRST, targetField);
        return this;
    }

    public GroupBuilder last(String targetField) {
        addOperand(GroupOperators.LAST, targetField);
        return this;
    }

    public GroupBuilder max(String targetField) {
        addOperand(GroupOperators.MAX, targetField);
        return this;
    }

    public GroupBuilder min(String targetField) {
        addOperand(GroupOperators.MIN, targetField);
        return this;
    }

    public GroupBuilder avg(String targetField) {
        addOperand(GroupOperators.AVG, targetField);
        return this;
    }

    public GroupBuilder push(String targetField) {
        addOperand(GroupOperators.PUSH, targetField);
        return this;
    }

    /**
     * Creates a <code>DBObject</code> to be passed to a $group operator
     *
     * @return Returns a DBObject instance
     * @throws RuntimeException if a key does not have a matching operand
     */
    public DBObject get() {
        for(String key : _body.keySet()) {
            if(_body.get(key) instanceof NullObject) {
                throw new GroupBuilderException("No operand for key: " + key);
            }
        }
        return _body;
    }

    @SuppressWarnings("serial")
	static class GroupBuilderException extends RuntimeException {
        GroupBuilderException(String message) {
            super(message);
        }
	}
    private static class NullObject {}

    private DBObject _body;
    private String _currentKey;
}
